package com.ematou.wxservice.service;

import com.alibaba.fastjson.JSON;
import com.ematou.wxservice.mp.message.WeChatMessage;
import com.ematou.wxservice.mp.handler.WeChatMessageHandler;
import com.ematou.wxservice.mp.message.WeChatMpXmlOutMessage;
import com.ematou.wxservice.mp.router.WeChatMessageHandlerRouter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

/**
 * @author lhm
 * @version 1.0
 * 2021-05-11 16:19
 */
@Service
public class WeChatMessageService {

    private static final Logger logger = LoggerFactory.getLogger(WeChatMessageHandlerRouter.class);

    /**
     * 处理消息
     * @param weChatMessage 微信推送的消息
     * @return  响应给用户的消息
     */
    public WeChatMpXmlOutMessage handleMessage(WeChatMessage weChatMessage) {
        logger.info("wxservice接收到微信平台推送的消息：" + JSON.toJSONString(weChatMessage));

        WeChatMessageHandler handler = WeChatMessageHandlerRouter.router(weChatMessage);

        if (null == handler) {
            logger.warn("暂不支持" + weChatMessage.getMsgType() + "类型或事件的处理");
            return null;
        }

        WeChatMpXmlOutMessage outMessage = handler.handlerMessage(weChatMessage);

        logger.info("wxservice处理完响应给用户的消息：" + JSON.toJSONString(outMessage));
        return outMessage;

    }

}
